@@ -23,7 +23,7 @@ def get_name():
     return "{}:{}".format(random.choice(name), random.randint(10000, 99999))
 
 
-def setup_p2p_params(network_ver, p2p_port, p2p_accept=True, sub_dir=None, f_debug=False):
+def setup_p2p_params(network_ver, p2p_port, p2p_accept=True, p2p_udp_accept=True, sub_dir=None, f_debug=False):
     if f_debug:
         Debug.P_EXCEPTION = True
         Debug.P_RECEIVE_MSG_INFO = True
@@ -50,6 +50,7 @@ def setup_p2p_params(network_ver, p2p_port, p2p_accept=True, sub_dir=None, f_deb
     V.NETWORK_VER = network_ver
     V.P2P_PORT = p2p_port
     V.P2P_ACCEPT = p2p_accept
+    V.P2P_UDP_ACCEPT = p2p_udp_accept
 
 
 def is_reachable(host, port):
@@ -60,6 +61,7 @@ def is_reachable(host, port):
         except OSError:
             continue
         result = sock.connect_ex(host_port)
+        sock.close()
         if result == 0:
             return True
     else:
